##Updated: January 28,2013


##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
## Programs (alphabetical order)
##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


## ---------------------------------------------------------------------------------
## bdate00.m
## This procedure estimates the model allowing for multiple breaks
## - - input - -
## y        [NTx1]: dependent variable
## x        [NTxp]: the matrix of regressors
## vec.tau        : the quantiles of interest
## n.size   [1x1] : the size of cross sections
## m        [1x1] : the maximum number of breaks 
## trim.size[1x1] : The trimming parameter (to exclude the boundary of the sample) 
## - - output - -
## mat.date [1x1] : the estimated break dates given 1-m breaks
## ---------------------------------------------------------------------------------

## When m = 1, partition(.) procedure is enough.
## When m > 1, this function applies the dynamic programming (DP)
## algorithm to look for the optimal breaks.

  
bdate00.m = function(y, x, n.size, m, trim.size, vec.long)
{
  ## size
  t.size = length(y) / n.size

  mat.date = matrix(0, m, m) 
  ## an upper-triangular matrix containing the estimated break dates 
  ## from one to m

  mat.opt.loc = matrix(0, t.size, m)
  ## a row index corresponding to the ending dates, column index corresponds 
  ## to the number of breaks permitted before the ending date the cell 
  ## contains the break date

  mat.opt.rho = matrix(0, t.size ,m)      
  ## same as above, the cell contains the minimal value of the objective 
  ## fucntion corresponding to that break date

  dvec = matrix(0, t.size,1)
  ## the index is the date after which we inserting the break point. The 
  ## cell contains the corresponding value of the objective function

  vec.global = matrix(0, m, 1)
  ## Global minimum when i breaks are permitted


  ## m = 1
  if (m == 1){
    res01 = partition(vec.long, 1, trim.size, t.size-trim.size, t.size, t.size) 
    mat.date[1,1] = res01$loc.min 
    vec.global[1] = res01$rho.min 
  } else {
    ## when m > 1, a dynamic programming algorithm is used.
    ## The first step is to obtain the optimal one-break partitions for all
    ## possible ending dates from 2h to T-mh+1.
    ## The optimal dates are stored in a vector optdat.
    ## The associated MML are stored in a vector optssr.


    ## First loop. Looking for the optimal one-break partitions for break 
    ## dates between h and T-h. j1 is the last date of the segment.
    for (j1 in (2*trim.size):t.size) { 
      res02 = partition(vec.long, 1, trim.size, (j1-trim.size), j1, t.size)           
      mat.opt.rho[j1,1] = res02$rho.min         # no typo
      mat.opt.loc[j1,1] = res02$loc.min
      
      vec.global[1] = mat.opt.rho[t.size,1]   # no typo
      mat.date[1,1] = mat.opt.loc[t.size,1]

    
      ## Next, the algorithm looks for optimal 2,3,... breaks partitions
      ## The index used is ib.
      for (ib in 2:m){
        if (ib == m){                 
          ## if we have reached the maximum number of breaks allowed,
          ## then do the following
          jlast = t.size
          
          beg01 = ib * trim.size
          end01 = t.size - trim.size
          for (jb in beg01:end01){
            dvec[jb] = mat.opt.rho[jb,ib-1] + vec.long[(jb+1)*t.size-jb*(jb+1)/2]
          }
          min.loc = (beg01 - 1) + which.min(dvec[beg01:end01])
          mat.opt.rho[jlast,ib] = dvec[min.loc]
          mat.opt.loc[jlast,ib] = min.loc
        } else {
          ## if we have not reached the highest number of breaks considered, 
          ## we need to loop over the possible last dates of the segments, between (ib+1)*h and T.
          for(jlast in ((ib+1)*trim.size):t.size){
            beg01 = ib * trim.size
            end01 = jlast - trim.size
            for(jb in beg01:end01){             
              dvec[jb] = mat.opt.rho[jb,ib-1] + vec.long[jb*t.size-jb*(jb-1)/2+jlast-jb]
            }
            min.loc = (beg01 - 1) + which.min(dvec[beg01:end01])
            mat.opt.rho[jlast,ib] = dvec[min.loc]
            mat.opt.loc[jlast,ib] = min.loc
          }
        }

       
        mat.date[ib,ib] = mat.opt.loc[t.size,ib]
        for(i in 1:(ib-1)){
          xx = ib - i
          mat.date[xx,ib] = mat.opt.loc[mat.date[xx+1,ib],xx]
        }
        vec.global[ib] = mat.opt.rho[t.size,ib]
      }
    }         # closing the if for the case m > 1

  }
  ## return
  return(mat.date)

}



## ---------------------------------------------------------------------------------
## ci.date.m
## This procedure constructs confidence intervals for the break dates
## based on multiple quantiles
## - - input - -
## y        [NTx1]    : dependent variable
## x        [NTx(p-1)]: regressors (excluding the constant term)
## vec.tau            : the quantiles of interest
## vec.date [mx1]     : break dates
## n.size   [1x1]     : sample size wrt the cross sectional dimension
## - - output - -
## mat.ci   [mx3]: 1st column includes break dates,
##                 2nd and 3rd columns include confidence intervals
## - - related sub routine - -
## rq.est.full, sample.split, moment. 
## ---------------------------------------------------------------------------------

ci.date.m = function(y, x, vec.tau, vec.date, n.size, v.b)
{
  ## size
  n.break = length(vec.date)
  n.tau   = length(vec.tau)
  p.size  = ncol(x) + 1    ## + intercept
  
  ## storage
  mat.ci     = matrix(0, n.break, 3) 
  mat.ci[,1] = matrix(vec.date, n.break, 1)

  ## break size
  mat.size = matrix(0, (n.break*p.size), n.tau)
  for(k in 1:n.tau){
    v.tau = vec.tau[k]
    vec.b = rq.est.full(y, x, v.tau, vec.date, n.size)$coef[]
    mat.size[,k] = vec.b[(p.size+1):((n.break+1)*p.size)]
  }
  
  ## confidence interval  
  ## 1st regime
  sub01  = sample.split(y, x, vec.date[1], n.size)
  y.L    = sub01$y1
  x.L    = sub01$x1
  y.rem  = sub01$y2
  x.rem  = sub01$x2

  ## IF the number of breaks is more than two. 
  if(n.break >= 2){
    for(i in 1:(n.break-1)){

      ## split sample
      date02 = vec.date[(i+1)] - vec.date[i] 
      sub02  = sample.split(y.rem, x.rem, date02, n.size)
      y.R    = sub02$y1
      x.R    = sub02$x1
      y.rem  = sub02$y2
      x.rem  = sub02$x2

      ## conficence interval for 1st - (m-1)th break
      v.date    = vec.date[i] # <== Put a real break date
      beg.row   = p.size * (i -1) + 1
      end.row   = p.size * i
      temp.size = as.matrix(mat.size[beg.row:end.row,], ncol = n.tau)
      mat.ci[i,] = ci.date.m.sub(y.L, x.L, y.R, x.R, v.date, temp.size, vec.tau, n.size, v.b)

      ## replace
      y.L = y.R
      x.L = x.R
    }
  }

  ## For the last break
  v.date    = vec.date[n.break] # <== Put a real break date
  beg.row   = p.size * (n.break -1) + 1
  end.row   = p.size *  n.break
  temp.size = as.matrix(mat.size[beg.row:end.row,], ncol = n.tau)
  mat.ci[n.break,] = ci.date.m.sub(y.L, x.L, y.rem, x.rem, v.date, temp.size, vec.tau, n.size, v.b)
  
  ## return
  return(mat.ci)
}




## ---------------------------------------------------------------------------------
## ci.date.m.sub
## This is a sub-rutine for ci.date.m
## - - input - -
## y.L, y.R [?x1]     : observations (y) on the right- and left-side of the break date 
## x.L, x.R [?x(p-1)] : observations (x) .... (excluding the constant)
## vec.tau  [kx1]     : the quantiles of interest
## v.date   [1x1]     : break dates
## n.size   [1x1]     : sample size wrt cross sectional dimension
## - - output - -
## vec.ci   [1x3]: 1st column includes break dates,
##                 2nd and 3rd columns include confidence intervals
## - - related sub routine - -
## sample.split, moment.
## ---------------------------------------------------------------------------------

ci.date.m.sub = function(y.L, x.L, y.R, x.R, v.date, mat.size, vec.tau, n.size, v.b)
{
  ## size
  n.tau = length(vec.tau)
  
  ## Adjustment to use the shrinking-break asymptotic framework
  mat.size = mat.size * sqrt(n.size)

  ## vec.size
  mat.L = matrix(0, n.tau, 2)
  mat.R = matrix(0, n.tau, 2)

  ## taus
  for(i in 1:n.tau){
    v.tau    = vec.tau[i]
    vec.size = mat.size[,i]
    ## moment
    mL  = moment(y.L, x.L, v.tau)
    mR  = moment(y.R, x.R, v.tau)
    ## pi
    mat.L[i,1] = t(vec.size) %*% mL$H %*% vec.size
    mat.R[i,1] = t(vec.size) %*% mR$H %*% vec.size
    ## sigma
    for(s in 1:n.tau){
      ## size for s
      vec.size.S = mat.size[,s]
      ## mix: Note, J does not depdent on tau
      temp = min(v.tau, vec.tau[s]) - v.tau * vec.tau[s]
      ## - - difference m1$J and m2$J 
      mat.L[i,2] = mat.L[i,2] + temp * t(vec.size) %*% mL$J %*% vec.size.S
      mat.R[i,2] = mat.R[i,2] + temp * t(vec.size) %*% mR$J %*% vec.size.S
    }
  }

  ## summation
  pi.L = sum(mat.L[,1])
  pi.R = sum(mat.R[,1])
  s2.L = sum(mat.L[,2])
  s2.R = sum(mat.R[,2])

  ## confidence interval
  vec.Q = c(7.7, 11.0)  ## 90% and 95% confidence levels
  vec.ci = matrix(0, 1, 3)
  vec.ci[1] = v.date
  vec.ci[2] = v.date - round( (vec.Q[v.b] * s2.L / (pi.L ^ 2) )) - 1
  vec.ci[3] = v.date + round( (vec.Q[v.b] * s2.R / (pi.R ^ 2) )) + 1

  return(vec.ci)
}



## ---------------------------------------------------------------------------------
## critical.DQtes_specific
## This procedure obtains critical values via simulation
## - - input - -
## x        [NTxp]: the matrix of regressors (not including an intercept)
## m.max    [1x1] : the maximum number of breaks 
## vec.tau        : the quantiles of interest
## - - output - -
## This procedure saves a text file in your working directory.
## The name of the file is "table.cv.DQ_t1_t2_p.txt",
## where
##   t1 = min(vec.tau), the minimum value of the quantiles; 
##   t2 = max(vec.tau), the maximum value of the quantiles;
##   p  = the number of regressors (including an intercept).
## ---------------------------------------------------------------------------------


## function
critical.DQtest_specific = function(x, m.max, vec.tau)
{
  ## start time
  time = proc.time()

  ## size
  p.size = ncol(x) + 1 ## include intercept
  
  ## setting
  n.grid   = 500
  n.sim    = 50000 ## In Qu(2009), 50,000

  ## significance level
  vec.a = c(0.90, 0.95, 0.99)
  n.a   = length(vec.a)

  ## probablity critical value
  vec.nn = seq(1, m.max, by = 1) 
  mat.prob = matrix(0, 3, m.max)
  mat.prob[1,] = 0.90 ^ (1 / vec.nn)
  mat.prob[2,] = 0.95 ^ (1 / vec.nn)
  mat.prob[3,] = 0.99 ^ (1 / vec.nn)
  mat.index = round(mat.prob * n.sim)

  ## discretization of a range of quantiles
  beg.tau   = min(vec.tau)
  end.tau   = max(vec.tau)
  cont.tau = seq(beg.tau, end.tau, by = 1 / n.grid)
  n.tau    = length(cont.tau)

  ## table
  table.cv     = matrix(0, 2, m.max)
  table.cv[1,] = beg.tau
  table.cv[2,] = end.tau

  ## seed
  set.seed(7, kind = NULL)

  ## simulation
  vec.p  = vec.prob.DQ(n.grid, p.size, n.sim, cont.tau)
  vec.p  = sort(vec.p)
  mat.cv   = matrix(0, 3, m.max)
  vec.name = matrix(p.size, 1, m.max)

  for(k in 1:3){
    mat.cv[k,] = vec.p[mat.index[k,]]
  }

  table.cv = rbind(table.cv, vec.name, mat.cv)

  ## save3
  file.name = paste('table.cv.DQ', beg.tau, '_', end.tau , '_', p.size, '.txt', sep='')
  write.table(table.cv,  file = file.name)

  ## end time
  cat('Time used for simulating the critical values',   proc.time() - time, '\n')
}



##--------------------------------------------------------------------
## dq.res.surface.r
## This procedure returns critical values using a response surface analysis.
## Please be advised that this procedure only covers
## the case where the trimming is symmetric, i.e., [tau,1-tau].

## - - input - -
## p        [NTx1]    : the number of parameters in the model
## l        [NTx(p-1)]: the number of changes under H0, i.e. l+1 under H1 
## q.L, q.R [1x1]     : [q.L, q.R] is the interval of quantiles
## - - output - - 
## vec.cv   [3x1]:     critical value
##--------------------------------------------------------------------

dq.res.surface.r = function(p, l, q.L, q.R)
{
  ## We need symmetry
  if (d.Sym != TRUE){
    stop('This program is only for DQ test with a symmetric trimming')
  }

  ## storage
  vec.cv = matrix(0, 3, 1)
  
  ## linear part 1
  vec.cv[1] = 0.9481 + 0.0062 * p + 0.0166 * (l + 1) -0.1386 * (1 / p)
  vec.cv[2] = 0.9944 + 0.0058 * p + 0.0157 * (l + 1) -0.1284 * (1 / p)
  vec.cv[3] = 1.0929 + 0.0050 * p + 0.0134 * (l + 1) -0.1134 * (1 / p)

  
  ## linear part 2
  vec.cv[1] = vec.cv[1] -0.0004 * (l+1) * p + 0.0018 * (l+1) * q.L
  vec.cv[2] = vec.cv[2] -0.0004 * (l+1) * p + 0.0017 * (l+1) * q.L
  vec.cv[3] = vec.cv[3] -0.0002 * (l+1) * p + 0.0010 * (l+1) * q.L

  ## exponantial part
  vec.cv[1] = vec.cv[1]*exp(-0.0801 * (1/(l+1)) -0.0004 * (1 /(q.L*(l+1))) -0.0254 * q.L)
  vec.cv[2] = vec.cv[2]*exp(-0.0716 * (1/(l+1)) -0.0005 * (1 /(q.L*(l+1))) -0.0203 * q.L)
  vec.cv[3] = vec.cv[3]*exp(-0.0565 * (1/(l+1)) -0.0000 * (1 /(q.L*(l+1))) -0.0062 * q.L)

  ## return
  return(vec.cv)
}



## ---------------------------------------------------------------------------------
## dq.test.0vs1
## This procedure tests for the existence of a single break
## over an interval of quantiles.
## Ho: no break
## H1: A single break exists
## - - input - -
## y        [NTx1]    : dependent variable
## x        [NTx(p-1)]: regressors (excluding the constant term). 
##                      All coefficients are allowed to change
## q.L, q.R [1x1]     : The lower and upper limits of the interval of quantiles
## n.size   [1x1]     : the size of cross sections
## - - output - - 
## v.max    [1x1]: test statistics 
## ---------------------------------------------------------------------------------

dq.test.0vs1 = function(y, x, q.L, q.R, n.size)
{
  ## size
  t.size = length(y) / n.size ## sub-sample size for time dim.

  ## regressors
  bigX    = cbind(1, x)
  p.size  = ncol(bigX)
  sqXX    = chol( t(bigX) %*% bigX)
  invsqXX = solve( t(sqXX) )

  ## triming
  seq.tau = seq(q.L, q.R, by = 1 / t.size)
  n.tau   = length(seq.tau)
  Qstat   = matrix(0, n.tau, 1)

  for(k in 1:n.tau){

    ## tau
    v.tau = seq.tau[k]

    ## fit
    res  = rq(y ~ 1 + x, tau = v.tau)$residuals
    temp = (res <= 0.0) - v.tau
    
    ## test
    H1n = invsqXX %*% t(bigX) %*% temp
    H1n = t(H1n)
    Hlambda = matrix(0, t.size, p.size)
    difH    = Hlambda
    for (t in 2:t.size) {
      HH = t(bigX)[,1:(t*n.size)] %*% temp[1:(t*n.size)]
      Hlambda[t,] = invsqXX %*% HH
      difH[t,] = Hlambda[t,] - (t / t.size) * H1n
    }

    ## maximum given v.tau
    Qstat[k] = max(abs(difH))
  }

  ## return
  return(max(Qstat))                                        
}



## ---------------------------------------------------------------------------------
## sq.test.lvsl_1
## This procedure tests for the existence of L breaks against (L+1) breaks
## over an interval of quantiles.
## Ho: L   breaks
## H1: L+1 breaks
## - - input - -
## y        [NTx1]: dependent variable
## x        [NTxp]: regressors
## q.L, q.R [1x1] : The lower and upper limits of the interval of quantiles
## n.size   [1x1] : the dimension of the cross section
## vec.date [Lx1] : break dates  
## - - output - - 
## v.max    [1x1]: test statistics 
## ---------------------------------------------------------------------------------

dq.test.lvsl_1 = function(y, x, q.L, q.R, n.size, vec.date)
{
  ## the number of break dates
  n.break  = length(vec.date)

  ## test for n.break regimes
  vec.test = matrix(0, (n.break+1), 1)
  rem.y  = y
  rem.x  = x
  pre.date = c(0, vec.date)
  for(j in 1:n.break){
    v.date = vec.date[j] - pre.date[j]
    ## split sample given a break date
    temp   = sample.split(rem.y, rem.x, v.date, n.size)
    rem.y  = temp$y2
    rem.x  = temp$x2
    ## test statistics
    vec.test[j] = dq.test.0vs1(temp$y1, temp$x1, q.L, q.R, n.size)
  }
  ## the last regime
  vec.test[(n.break+1)] = dq.test.0vs1(rem.y, rem.x, q.L, q.R, n.size)
  
  ## return: maximum over regimes
  return( max(vec.test) ) 
}




## ------------------------------------------------
## gen.long.rho.r
## to obtain the objective function values
## given permissible regimes.
## - - input - -
## y        [NTx1]: dependent variable
## x        [NTxp]: regressors whose coefficients have changes
## vec.tau        : quantiles of interest
## n.size   [1x1] : the size of cross sections
## trim.size[1x1] : triming size 
## - - output - -
## mat.rho        : the estimated break date 
## ------------------------------------------------

gen.long = function(y, x, vec.tau, n.size, trim.size)
{
  ## size
  t.size = length(y) / n.size
  n.tau  = length(vec.tau)

  ## the following matrix and vector contain the objective function
  ## corresponding to all the possible segments. The value corresponds to the
  ## segment starting at J and lasting for span corresponds to the index
  ## T(j-1)-(j-1)(j-2)/2+span. The matrix and vector are for single quantile and
  ## the combined quantiles, respectively. 
  mat.long = matrix(0, (t.size*(t.size+1)/2), n.tau)  ## individual quantiles  
  vec.long = matrix(0, (t.size*(t.size+1)/2), 1)      ## multiple   quantiles

  for(i in 1:(t.size-trim.size+1)){
    mat.out = gen.mat.rho(y, x, vec.tau, n.size, trim.size, i, t.size)
  
    ## rows
    row01 = (i-1) * t.size + i - (i - 1) * i / 2
    row02 =  i    * t.size     - (i - 1) * i / 2

    ## store
    mat.long[row01:row02,] = mat.out[i:t.size,]
    vec.long[row01:row02]  = rowSums(mat.out[i:t.size,])

  }
  

  ## return
  list(mat.long = mat.long, vec.long = vec.long)
}



## ------------------------------------------------
## gen.mat.rho.r
## This procedure obtains the values of the objective function
## given permissible partitions.
## - - input - -
## y        [NTx1]: dependent variable
## x        [NTxp]: regressors whose coefficients have changes
## vec.tau        : quantiles of interest
## n.size   [1x1] : the size of cross sections
## trim.size[1x1] : triming size 
## - - output - -
## mat.rho        : the estimated break date 
## ------------------------------------------------

gen.mat.rho = function(y, x, vec.tau, n.size, trim.size, start, last)
{
  ## size
  n.tau = length(vec.tau)

  ## storage
  mat.rho = matrix(0, last, n.tau)

  ## quantile regression
  col01 = n.size * (start - 1) + 1
  loc01 = start + trim.size - 1      ## the end of sample: start
  loc02 = last                       ## the end of sample: final
  for(j in loc01:loc02){
    col02 = n.size * j

    ## calcuate rho for each quntile
    temp.rho = matrix(0, n.tau, 1)
    for(k in 1:n.tau){
      y.temp = y[col01:col02]
      x.temp = x[col01:col02,]
      mat.rho[j,k] = rq(y.temp ~ x.temp, tau = vec.tau[k])$rho
    }
  }

  ## return
  return(mat.rho)
}



## ---------------------------------------------------------------------------------
## get.cv.dq.r
## Critical values for DQ(l|l+1)
## - - input - -
## n.break [1x1]: the number of breaks under H0
## p.size  [1x1]: the number of regressors 
## - - output - -
## vec.cv  [3x1]: critical values at 10%, 5%, 1% significance level 
## ---------------------------------------------------------------------------------

get.cv.dq = function(n.break, p.size, q.L, q.R)
{
  
  #row.beg = 4 * (p.size-1) + 4 # the first three lines include q.L, q.R &  #test
  #row.end = 4 * (p.size-1) + 6 
  #vec.cv  = mat.cv[row.beg:row.end, n.break]

  if (d.Sym == TRUE){

    ## response surface
    vec.cv = dq.res.surface.r(p.size, n.break, q.L, q.R)

  } else {
    

    file.name = paste('table.cv.DQ', q.L, '_', q.R , '_', p.size, '.txt', sep='')
    cat('Note: DQ test uses critical vlues from', file.name, ' \n')
    mat.cv = as.matrix(read.table(file.name, header=T), ncol = 3)
    vec.cv = mat.cv[4:6,(n.break+1)] 

  }
  
  ## return
  return(vec.cv)
}



## ---------------------------------------------------------------------------------
## get.cv.sq
## Critical values for SQ(l|l+1)
## - - input - -
## n.break [1x1]: the number of breaks under H0
## k.size  [1x1]: the number of regressors 
## - - output - -
## vec.cv  [3x1]: critical values at 10%, 5%, 1% significance level 
## ---------------------------------------------------------------------------------

get.cv.sq = function(n.break, p.size)
{
  mat.cv = as.matrix(read.table("table.cv.SQtest010910.txt", header=T), ncol = 5)
  
  row.beg = 4 * (p.size-1) + 1 + 1 # the first line includes the number of test 
  row.end = 4 * (p.size-1) + 4

  vec.cv  = mat.cv[row.beg:row.end, (n.break+1)]

  ## return
  return(vec.cv)
}



## ---------------------------------------------------------------------------------
## moment
## This procedure obtains relevant quantities for constructing the confidence intervals  
## - - input - -
## y        [NTx1] : dependent variable
## x        [NTxp] : regressors 
## v.tau    [1x1]  : the (single) quantile of interest
## - - output - -
## H
## J
## mean.f
## ---------------------------------------------------------------------------------

moment = function(y, x, v.tau)
{
  ## size
  nt.size = length(y)

  ## regressor
  x.1 = cbind(1, x)

  ## bandwidth 
  h = bandwidth.rq(v.tau, nt.size, hs = FALSE)
  if (v.tau + h > 1) 
    stop("v.tau + h > 1:  error in summary.rq")
  if (v.tau - h < 0) 
    stop("v.tau - h < 0:  error in summary.rq")

  ## quotient
  bhi = rq.fit.fnb(x.1, y, tau = v.tau + h)$coef
  blo = rq.fit.fnb(x.1, y, tau = v.tau - h)$coef
  dyhat = x.1 %*% (bhi - blo)
  if (any(dyhat <= 0)) {
    warning(paste(sum(dyhat <= 0), "non-positive fis"))
  }
  f = pmax(0, (2 * h)/(dyhat - eps))
  H = (1 / nt.size) * crossprod( (f * x.1), x.1) 
  J = (1 / nt.size) * crossprod(x.1)
  mean.f = mean(f)

  ## return
  list(H = H, J = J, mean.f = mean.f)
}




## ------------------------------------------------
## partition.r
## procedure to obtain an optimal one break partition
## a segment, [start,end].  
## 
## - - input - -
## vec.long      : a long vector contains the values of the objective function 
## start    [1x1]: the beginning of the segment considered
## b1       [1x1]: the first possible break date
## b2       [1x1]: the last  possible break date
## last     [1x1]: the end of segment considered 
## t.size   [1x1]: the size of the cross section
## - - output - -
## loc.min  [1x1]: an optimal break point
## rho.min  [1x1]: the value of objective function
## ------------------------------------------------

partition = function(vec.long, start, b1, b2, last, t.size)
{
  ## initial 
  ini = (start - 1) * t.size - (start - 2) * (start - 1) / 2 + 1

  ## storage
  temp.rho = matrix(0, t.size, 1)

  ## pick up vec of rho
  for(j in b1:b2){
    loc01 = j - start + ini
    loc02 = j * t.size - (j - 1) * j / 2 + last - j

    ## 1st term: rho of [start,j]
    ## 2nd term: rho of [(j+1),last-j]
    temp.rho[j] = vec.long[loc01] + vec.long[loc02]
  }

  ## minimization
  loc.min = (b1 - 1) + which.min(temp.rho[b1:b2])
  rho.min = temp.rho[loc.min]

  ## return
  list(loc.min = loc.min, rho.min = rho.min)
}

## ---------------------------------------------------------------------------------
## rq.break
## This procedure combines the subroutines and returns the results
## - - input - -
## y        [NTx1]: dependent variable
## x        [NTxp]: the matrix of regressors
## vec.tau        : the quantiles of interest
## n.size   [1x1] : the size of cross sections
## trim.e   [1x1] : the trimming proportion when estimating the break dates
## q.L, q.R [1x1] : the quantile range [q.L, q.R] is used for DQ test
## vec.time [Tx1] : the time label for time-series domain
## m.max    [1x1] : the maximum number of breaks allowed
## ---------------------------------------------------------------------------------

rq.break = function(y, x, vec.tau, n.size, trim.e, q.L, q.R, vec.time, m.max, v.a, v.b)
{

  ## size
  N.tau     = length(vec.tau)
  T.size    = length(y) / n.size    # size of time series
  trim.size = round(T.size * trim.e)
  p.size = ncol(x) + 1  # plus intercept

  ##===========================================================
  ## Analysis Starts from Here
  ##===========================================================

  ## starting time
  time = proc.time()

  ## simulation to generate critical values 
  if (d.Sym == FALSE & d.Sim == TRUE){
    critical.DQtest_specific(x, m.max, vec.tau)
  }

  ## obtain values of objective function 
  out.long   = gen.long(y, x, vec.tau, n.size, trim.size)
  mat.long.s = out.long$mat.long  ## for individual quantiles
  vec.long.m = out.long$vec.long  ## for mulitple   quantiles
  
  ## Single Quantile
  cat('================================================================== \n')
  cat('=====Analysis based on a single conditional quantile function ===== \n')

  ## storage
  vec.nb      = matrix(0, (N.tau*3), 1)
  mat.loc.opt = matrix(0, (N.tau*3), m.max)
  
  for(i in 1:N.tau){    

    ## tau 
    v.tau  = vec.tau[i]
    
    cat('================================================================== \n')
    cat('Quantile: ', v.tau, '\n')

    ## break dates given the maximum number of breaks
    vec.long.s = mat.long.s[,i]
    mat.date   = bdate00.m(y, x, n.size, m.max, trim.size, vec.long.s)
    ## print(mat.date)

    ## determine the number of breaks
    out.s = seq.sq(y, x, v.tau, n.size, m.max, trim.size, mat.date)
    
    ## estimation and inference
    vec.level = c(10, 5, 1)
    
    ## print out:
    cat('------------- At the significance level: ', vec.level[v.a],'% -------------\n' )
    cat(' SQ test:              ', out.s$test,   '\n')
    cat(' Critical values:      ', out.s$cv[v.a,],    '\n')
    cat(' The number of breaks detected: ', out.s$nbreak[v.a], '\n')
    
    ## estimation
    n.break      = out.s$nbreak[v.a]
    ind01        = i + N.tau * (v.a - 1) ## store
    vec.nb[ind01]= n.break             ## store
    if (n.break >= 1){
      vec.date = out.s$date[v.a,1:n.break]
      fit      = rq.est.full(y, x, v.tau, vec.date, n.size)
      result   = summary.rq(fit, se = "nid", covariance = TRUE)
      mat.ci   = ci.date.m(y, x, v.tau, vec.date, n.size, v.b)
      cat(' - - Break Dates and Confidence Intervals: ', 100-vec.level[v.b], '% - -     \n')
      print( mat.ci)
      if (min(mat.ci[,2]) < 1 | T.size < max(mat.ci[,3])){
        cat('  confidence interval is out of the range\n') 
      } else {
        print( matrix( vec.time[mat.ci], ncol = 3) )
      }

      cat(' - - Estimation Results - - \n')
      
      ## Estimates for each regime
      cat(' (a)Coefficients estimates for each regime - - \n')
      rq.est.regime(y, x, v.tau, vec.date, n.size)

      ## Estimates for the break sizes
      cat(' (b) Break sizes - - \n')
      
      for (j in 1:n.break){
        beg01 = 1 + p.size * j 
        end01 =     p.size * (j+1) 
        print(result$coef[beg01:end01,])
        cat('  \n') # space
      }

      ## store
      mat.loc.opt[ind01,1:n.break] = vec.date
    } 
  }      

  ## figure for single quantile
  ## fig.s(y, x, vec.tau, vec.nb, mat.date, vec.time, n.size)

  
  ## Multiple quantiles
  cat('================================================================== \n')
  cat('===== Analysis based on multiple conditional quantiles  ===== \n')
  cat('================================================================== \n')

  ## experiment
  mat.date = bdate00.m(y, x, n.size, m.max, trim.size, vec.long.m)
  ## print(mat.date)
  ## (i) determine the number of breaks
  out.m = seq.dq(y, x, vec.tau, q.L, q.R, n.size, m.max, trim.size, mat.date)

  vec.level = c(10, 5, 1) # significance level


  ## the number of breaks
  n.break = out.m$nbreak[v.a]

  ## print 
  cat('------------- At the significance level: ', vec.level[v.a], '% -------------\n' )
  cat(' DQ test:              ', out.m$test,  '\n')
  cat(' Critical values:      ', out.m$cv[v.a,],    '\n')
  cat(' The number of breaks detected: ', n.break,         '\n')

  if (n.break >= 1){
    vec.date = out.m$date[v.a,1:n.break]
      ## confidence interval
    mat.ci = ci.date.m(y, x, vec.tau, vec.date, n.size, v.b)
    cat(' - - Break Dates and Confidence Intervals: ', 100-vec.level[v.b], '% - -     \n')
    if (min(mat.ci[,2]) < 1 | T.size < max(mat.ci[,3])){
      cat('  confidence interval is out of the range\n') 
    } else {
      print(mat.ci)
    }
    print( matrix( vec.time[mat.ci], ncol = 3) )
    cat('- - - - Estimation Results - - - -\n')
    for(k in 1:N.tau){
      v.tau  = vec.tau[k]
      cat('- Quantile: ', v.tau ,'-\n')

      ## Estimates for each regime by splitting sample
      cat(' (a) Coefficients estimates for each regime - - \n')
      rq.est.regime(y, x, v.tau, vec.date, n.size)


      ## Estimates of the break sizes
      cat(' (b) Break sizes - - \n')
        
      fit    = rq.est.full(y, x, v.tau, vec.date, n.size)
      result = summary.rq(fit, se = 'nid', covariance = TRUE)

      for (j in 1:n.break){
        beg01 = 1 + p.size * j 
        end01 =     p.size * (j+1) 
        print(result$coef[beg01:end01,])
        cat('  \n') # space
      }
        
    }

  }

  cat('================================================================== \n')
  ## end
  cat('Estimation Time (min) \n')
  print( (proc.time() - time) / 60)
  cat('================================================================== \n')
}




## ---------------------------------------------------------------------------------
## rq.est.full.r
## This procedure estimates the coefficients conditional on estimated break dates.
## Note: the results are in terms of break sizes
## - - input - -
## y        [NTx1] : dependent variable
## z        [NTxp] : regressors
## vec.tau         : the quantiles of interest  
## vec.date [n.bx1]: break dates, where n.b = n.break
## v.tau    [1x1]  : the (single) quantile of interest
## n.size   [1x1]  : the size of cross sections
## - - output - -
## fit : object from rq() 
## ---------------------------------------------------------------------------------

rq.est.full = function(y, x, v.tau, vec.date, n.size)
{
  ## add an intercept
  x.1 = cbind(1, x)

  ## size and # breaks
  nt.size = length(y)
  t.size  = nt.size / n.size
  p.size  = ncol(x.1)
  n.break = length(vec.date)

  ## note: matrix looks like stairs to obtain break sizes
  b.size = matrix(0, nt.size, (n.break*p.size)) 
  for(i in 1:n.break){
    r.beg = n.size * vec.date[i] + 1
    c.beg = p.size * (i -1) + 1
    c.end = p.size *  i
    b.size[r.beg:nt.size,c.beg:c.end] = x.1[r.beg:nt.size,]
  }
  big.x = cbind(x, b.size)
  
  ## estimation
  fit = rq(y ~ big.x, tau = v.tau)   ## w/o intercepts b/c rq() adds it

  ## return
  return(fit)
}



## ------------------------------------------------
## rq.est.regimie.r
## This procedure estimates the coefficients for each regime, given the break dates.
## It
## a)  splits the sample given break dates;
## b) estimates a model for each regime separately 
## "rq.est.full.r" return break sizes
## - - input - -
## y        [NTx1] : dependent variable
## z        [NTxp] : regressors whose coefficients have changed
## vec.date [n.bx1]: break dates, where n.b = n.break
## v.tau    [1x1]  : quantile of interest
## n.size   [1x1]  : the size of the cross section
## - - output - -
## fit : object from rq() 
## ------------------------------------------------

rq.est.regime = function(y, x, v.tau, vec.date, n.size)
{

  ## the number of breaks 
  n.break  = length(vec.date)

  ## sequationally split sample 
  rem.y  = y
  rem.x  = x
  pre.date = c(0, vec.date)
  for(j in 1:n.break){
    cat('  < Regime:', j,'>\n')
    v.date = vec.date[j] - pre.date[j]
    ## split sample given a break date
    temp   = sample.split(rem.y, rem.x, v.date, n.size)
    rem.y  = temp$y2
    rem.x  = temp$x2
    ## estimation
    fit.regime  = rq(temp$y1 ~ temp$x1, v.tau)
    sum.regime  = summary.rq(fit.regime, se = "nid", covariance = TRUE)

    ## print results
    print(sum.regime$coef)
    cat('\n') ## space
  }

  ## the last regime
  cat('  < Regime:', (n.break + 1), '>\n')
  fit.regime  = rq(rem.y~rem.x, v.tau)
  sum.regime  = summary.rq(fit.regime, se = "nid", covariance = TRUE)
  print(sum.regime$coef)
  cat('\n') ## space
}



## ---------------------------------------------------------------------------------
## sample.split
## This procedure splits the sample into two 
## - - input - -
## y        [NTx1]: dependent variable
## x        [NTxp]: regressors 
## v.date   [1x1] : break date
## n.size   [1x1] : the size of cross sections
## - - output - - 
## yi, xi: subsamples, where i = 1 or 2
## ---------------------------------------------------------------------------------

sample.split = function(y, x, v.date, n.size)
{
  ## sample size and the location to split
  nt.01   = v.date * n.size ## the end of a regime
  nt.size = length(y)

  ## split 
  y1 = as.matrix( y[1:nt.01]  )
  x1 = as.matrix( x[1:nt.01,] )  
  y2 = as.matrix( y[(nt.01+1):nt.size]  )
  x2 = as.matrix( x[(nt.01+1):nt.size,] ) 

  ## return
  list(y1 = y1, x1 = x1, y2 = y2, x2 = x2)
}



## ---------------------------------------------------------------------------------
## seq.dq
## This procedure determines the number of breaks using DQ(l|l+1) test
## - - input - -
## y         [NTx1]: dependent variable
## x         [NTxp]: regressors 
## vec.tau         : quantiles of interest
## q.L, q.R  [1x1] : the quantile range [q.L, q.R] is used for DQ test
## n.size    [1x1] : cross-sectional sample size
## m.max     [1x1] : the maximum number of breaks allowed
## trim.size [1x1] : trimming 
## - - output - - 
## test   [1xm.max]: DQ test 
## cv     [3xm.max]: critical values for DQ test; 1st, 2nd & 3rd row = 10, 5, 1% size
## date   [3xm.max]: break dates;                 1st, 2nd & 3rd row = 10, 5, 1% size 
## nbreak [3x1]    : the number of breaks;        1st, 2nd & 3rd row = 10, 5, 1% size 
## - - related source codes - - 
## bdate01.m, bdate02.m, bdate03.m, get.cv.sq, dq.test.0vs1, dq.test.lvsl_1
## ---------------------------------------------------------------------------------

seq.dq = function(y, x, vec.tau, q.L, q.R, n.size, m.max, trim.size, mat.date)
{
  ## the number of regressors
  p.size = ncol(x) + 1  # plus intercept

  ## stroage
  vec.nb   = matrix(0, 3, 1)
  vec.test = matrix(0, 1, m.max)
  mat.cv   = matrix(0, 3, m.max)

  ## 0 vs 1
  vec.test[1] = dq.test.0vs1(y, x, q.L, q.R, n.size)
  mat.cv[,1]  = get.cv.dq(0, p.size, q.L, q.R) # critical value
  for (a in 1:3){
    if (mat.cv[a,1] < vec.test[1]){
      vec.nb[a] = 1
    }
  }

  ## k vs k+1
  if (m.max >= 2){
    for (k in 1:(m.max-1) ) { # k is the number of breaks
      if (max(vec.nb) == k){  
    
        ## 1st-kth break dates are stored in kth column 
        vec.loc = mat.date[1:k,k]  

        ## test: k vs k+1
        vec.test[(k+1)] = dq.test.lvsl_1(y, x, q.L, q.R, n.size, vec.loc)
        mat.cv[,(k+1)]  = get.cv.dq(k, p.size, q.L, q.R) # critical value

        ## for each significance level
        for (a in 1:3){
          if (vec.nb[a] == k  &  mat.cv[a,(k+1)] < vec.test[(k+1)]){
            vec.nb[a] = vec.nb[a] + 1
          }
        }
      }
    }
  }

  ## store optimal break dates
  mat.date.opt = matrix(0, 3, m.max)
  for(a in 1:3){
    nb = vec.nb[a]
    if (nb >=1){
      mat.date.opt[a,1:nb] = mat.date[1:nb,nb]
    }
  }

  ## return
  list(test = vec.test, cv = mat.cv, date = mat.date.opt, nbreak = vec.nb)
}



## ---------------------------------------------------------------------------------
## seq.sq
## To determine the number of breaks using the SQ(l|l+1) test
## - - input - -
## y         [NTx1]: dependent variable
## x         [NTxp]: regressors 
## v.tau          : the quantiles of interest
## n.size    [1x1] : the cross-sectional sample size
## m.max     [1x1] : the maximum number of breaks allowed
## trim.size [1x1] : trimming size 
## - - output - - 
## test   [1xm.max]: DQ test 
## cv     [3xm.max]: critical values for DQ test; 1st, 2nd & 3rd row = 10, 5, 1% size
## date   [3xm.max]: break dates;                 1st, 2nd & 3rd row = 10, 5, 1% size 
## nbreak [3x1]    : the number of breaks;        1st, 2nd & 3rd row = 10, 5, 1% size 
## - - related source codes - - 
## bdate01.m, bdate02.m, bdate03.m, get.cv.sq, sq.test.0vs1, sq.test.lvsl_1 
## ---------------------------------------------------------------------------------

seq.sq = function(y, x, v.tau, n.size, m.max, trim.size, mat.date)
{
  ## the number of regressors
  p.size = ncol(x) + 1  # plus intercept

  ## stroage
  vec.nb       = matrix(0, 3, 1)
  vec.test     = matrix(0, 1, m.max)
  mat.cv       = matrix(0, 3, m.max)

  ## 0 vs 1
  vec.test[1] = sq.test.0vs1(y, x, v.tau, n.size)
  mat.cv[,1]  = get.cv.sq(0, p.size) # critical value
  for (a in 1:3){
    if (mat.cv[a,1] < vec.test[1]){
      vec.nb[a] = 1
    }
  }

  ## k vs k+1
  if (m.max >= 2){
    for(k in 1:(m.max-1)){
      if (max(vec.nb) == k){

        ## 1st-kth break dates are stored in kth column 
        vec.loc = mat.date[1:k,k]  

        ## test: k vs k+1
        vec.test[(k+1)] = sq.test.lvsl_1(y, x, v.tau, n.size, vec.loc)
        mat.cv[,(k+1)]  = get.cv.sq(k, p.size) # critical value
        
        ## for each significance level
        for (a in 1:3){
          if (vec.nb[a] == k & mat.cv[a,(k+1)] < vec.test[(k+1)]){
            vec.nb[a] = vec.nb[a] + 1
          }
        }
      }
    }
  }
  ## store optimal break dates
  mat.date.opt = matrix(0, 3, m.max)
  for(a in 1:3){
    nb = vec.nb[a]
    if(nb >= 1){ 
      mat.date.opt[a,1:nb] = mat.date[1:nb,nb]
    }
  }

  ## return
  list(test = vec.test, cv = mat.cv, date = mat.date.opt, nbreak = vec.nb)
}



## ---------------------------------------------------------------------------------
## sq.test.0vs1
## To test the existence of a break in a single conditional quantile
## Ho: no break
## H1: A single break exists
## - - input - -
## y        [NTx1]    : dependent variable
## x        [NTx(p-1)]: regressors 
## v.tau    [1x1]     : quantile of interest
## n.size   [1x1]     : the size of cross sections
## - - output - - 
## v.max    [1x1]: test statistics 
## ---------------------------------------------------------------------------------

sq.test.0vs1 = function(y, x, v.tau, n.size)
{
  ## time dimension
  t.size = length(y) / n.size

  ## regressors
  bigX    = cbind(1, x)
  sqXX    = chol( t(bigX) %*% bigX)
  invsqXX = solve( t(sqXX) )

  ## quantile regression using all sample
  res = rq(y ~ x, tau = v.tau)$res 

  ## Test
  temp = (res <= 0.0) - v.tau  
  H1n  = invsqXX %*% t(bigX) %*% temp
  difH = matrix(0, 1, t.size)
  ## loop
  for (j in 2: t.size){
    end1 = n.size * j
    HH = t(bigX)[,1:end1] %*% temp[1:end1]
    difH[j] = max( abs(invsqXX %*% HH - (j / t.size) * H1n) )
  }

  v.max = max(difH) / sqrt( v.tau * (1 - v.tau) )

  ## Result
  return( v.max )
}



## ---------------------------------------------------------------------------------
## sq.test.lvsl_1.r
## To test the existence of (L+1) breaks against L breaks in a single conditional quantile
## Ho: l   breaks
## H1: l+1 breaks
## - - input - -
## y        [NTx1]: dependent variable
## x        [NTxp]: regressors 
## v.tau    [1x1] :
## n.size   [1x1] : the size of the cross section
## vec.date [lx1] : break dates  
## - - output - - 
## v.max    [1x1]: test statistic 
## ---------------------------------------------------------------------------------

sq.test.lvsl_1 = function(y, x, v.tau, n.size, vec.date)
{
  ## the number of breaks 
  n.break  = length(vec.date)

  ## sequential test
  vec.test = matrix(0, (n.break+1), 1)
  rem.y  = y
  rem.x  = x
  pre.date = c(0, vec.date)
  for(j in 1:n.break){
    v.date = vec.date[j] - pre.date[j]
    ## split sample given a break date
    temp   = sample.split(rem.y, rem.x, v.date, n.size)
    rem.y  = temp$y2
    rem.x  = temp$x2
    ## test statistics
    vec.test[j] = sq.test.0vs1(temp$y1, temp$x1, v.tau, n.size)
  }
  ## the last regime
  vec.test[(n.break+1)] = sq.test.0vs1(rem.y, rem.x, v.tau, n.size)
  
  ## return: maximum over regimes
  return( max(vec.test) ) 
}




## ---------------------------------------------------------------------------------
## vec.prob.DQ.r
## This is a subroutine to generate critical values for the DQ test
## - - input - -
## n.grid  [1x1]: the number of grid points
## n.dim   [1x1]: the number of regressors 
## n.sim   [1x1]: the number of repetition 
## vec.tau      : quantiles of interest
## ---------------------------------------------------------------------------------

vec.prob.DQ = function(n.grid, n.dim, n.sim, vec.tau)
{
  ## size
  n.tau = length(vec.tau)
  
  ## simulation
  vec.lamda = seq(0, 1, length = n.grid)
  vec.p     = matrix(0, n.sim, 1)
  for(s in 1:n.sim){

    ## Over dimenstion
    vec.max = matrix(0, n.dim, 1)
    for(d in 1:n.dim){

      ## e ~ U[0,1]
      vec.e = runif(n.grid, 0, 1)


      vec.limit = matrix(0, n.tau, 1)
      for(i in 1:n.tau){

        ## value tau
        v.tau = vec.tau[i]

        ## brownian motion
        vec.bm = cumsum( (vec.e <= v.tau) )

        ## brownian bridge
        vec.BB = (vec.bm - vec.lamda * vec.bm[n.grid]) / sqrt(n.grid)

        ## max over lamda
        vec.limit[i] = max(abs(vec.BB))
      }

      ## max over tau
      vec.max[d] = max(vec.limit)
    }      

    ## maximization over dimension
    vec.p[s] = max(vec.max)
  }

  return(vec.p)
}

###DONE!:)


